package com.gtf.template;

import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;

import java.util.Map;

/**
 * 抽象角色模版
 */
@Slf4j
public abstract class AbstractPayCallTemplate  {

    /**
     * 共同行为骨架
     * @return
     */
    public String asyncCall(){
        //1。验证参数和签名
        Map<String,String> verifySignature=verifySignature();
        //2.日志收集
        payLog(verifySignature);
        if (!verifySignature.get("analysisCode").equals("200")) {
            return resultFail();
        }
        //3.更改数据库状态，同时返回不同的支付结果。
         return  asyncService(verifySignature);
    }

    /**
     * 返回失败方法
     * @return
     */
    protected abstract String resultFail();

    /**
     * 使用多线程异步写入日志
     * @param verifySignature
     */
    @Async
    public void payLog(Map<String, String> verifySignature) {
        System.out.println("写入数据库中");
    }

    /**
     * 验证参数
     * @return
     */
    protected abstract Map<String, String> verifySignature();

    /**
     * 执行订单修改，返回不同的结果
     * @param verifySignature
     * @return
     */
    protected abstract String asyncService(Map<String, String> verifySignature);

    /**
     * 返回成功
     * @return
     */
    protected abstract String resultSuccess();
}
